"""
All Environment variables
"""
import click
from questionary import Separator
from questionary import Style

NOT_VALID_JWT = click.style("Enter a valid JWT!!!", fg="red", bold=True)
CHECK_DOCS = click.style(
    "Check Docs!!Your jwt have not a HS alg.",
    fg="red",
    bold=True,
)
NOT_CRAKED = click.style("JWT not cracked sorry. :'(", fg="red", bold=False)
CRACKED = click.style("JWT cracked, key is: ", fg="green", bold=False)
VALID_PAYLOAD = click.style(
    "Enter a Valid payload, Format: username=admin",
    fg="red",
    bold=False,
)
VALID_COOKIES = click.style(
    "Enter a Valid cookie, Format: username=admin",
    fg="red",
    bold=False,
)
VALID_DATA = click.style(
    "Enter a Valid data, Format: username=admin",
    fg="red",
    bold=False,
)
VALID_HEADER = click.style(
    "Enter a Valid header, Format: username=admin",
    fg="red",
    bold=False,
)
VALID_PAYLOAD_JSON = click.style(
    "Not a valid format for payload, send a json.",
    fg="red",
    bold=False,
)
NEW_JWT = click.style("new JWT: ", fg="green", bold=True)
VALID_SIGNATURE = click.style("Valid Signature!!", fg="green", bold=True)
INVALID_SIGNATURE = click.style("Incorrect signature!!", fg="red", bold=True)
CLIPBOARD = click.style("New jwt Copied to clipboard", fg="blue", bold=False)
# User interface
custom_style_fancy = Style(
    [
        ("qmark", "fg:#673ab7 bold"),  # token in front of the question
        ("question", "bold"),  # question text
        (
            "answer",
            "fg:#f44336 bold",
        ),  # submitted answer text behind the question
        (
            "pointer",
            "fg:#673ab7 bold",
        ),  # pointer used in select and checkbox prompts
        (
            "highlighted",
            "fg:#673ab7 bold",
        ),  # pointed-at choice in select and checkbox prompts
        ("selected", "fg:#cc5454"),  # style for a selected item of a checkbox
        ("separator", "fg:#cc5454"),  # separator in lists
        (
            "instruction",
            "",
        ),  # user instructions for select, rawselect, checkbox
        ("text", ""),  # plain text
        (
            "disabled",
            "fg:#858585 italic",
        ),  # disabled choices for select and checkbox prompts
    ],
)
# Summary
MAIN_SUMMARY_QUESTION = "What do you want to do?"
MAIN_SUMMARY_CHOICES_MODIFY = "Modify your jwt"
MAIN_SUMMARY_CHOICES_NONE_ALG = "Check None algorithm"
MAIN_SUMMARY_CHOICES_RSA_CONFUSION = "Check Rsa/Hmac confusion"
MAIN_SUMMARY_CHOICES_BRUTE_FORCE = (
    "Brute-force your jwt to guess key(wordlist needed)"
)
MAIN_SUMMARY_CHOICES_SIGN = "Sign your jwt"
MAIN_SUMMARY_CHOICES_VERIFY = "Verify your key"
MAIN_SUMMARY_CHOICES_KID = "Kid injection"
MAIN_SUMMARY_CHOICES_JKU = "Jku bypass"
MAIN_SUMMARY_CHOICES_X5U = "X5u bypass"
MAIN_SUMMARY_CHOICES_QUIT = "Quit"

# Separator
SEPARATOR_MODIFY_JWT = Separator(
    "--------------------------- Modify your jwt ---------------------------",
)
SEPARATOR_VULNERABILITIES = Separator(
    "---------------------------- Vulnerabilities ----------------------------",
)
SEPARATOR_GUESS_KEY = Separator(
    "------------------------------ Guess key ------------------------------",
)
SEPARATOR_ADVANCED = Separator(
    "------------------------------- Advanced -------------------------------",
)
SEPARATOR_QUIT = Separator(
    "--------------------------------- Quit ---------------------------------",
)

# choices

MAIN_SUMMARY_CHOICES = [
    SEPARATOR_MODIFY_JWT,
    MAIN_SUMMARY_CHOICES_MODIFY,
    # vulnerability
    SEPARATOR_VULNERABILITIES,
    MAIN_SUMMARY_CHOICES_NONE_ALG,
    MAIN_SUMMARY_CHOICES_RSA_CONFUSION,
    # guess key
    SEPARATOR_GUESS_KEY,
    MAIN_SUMMARY_CHOICES_BRUTE_FORCE,
    MAIN_SUMMARY_CHOICES_SIGN,
    MAIN_SUMMARY_CHOICES_VERIFY,
    # advanced
    SEPARATOR_ADVANCED,
    MAIN_SUMMARY_CHOICES_KID,
    MAIN_SUMMARY_CHOICES_JKU,
    MAIN_SUMMARY_CHOICES_X5U,
    SEPARATOR_QUIT,
    MAIN_SUMMARY_CHOICES_QUIT,
]
MAIN_SUMMARY_PROMPT_PEM = "Please enter your public key (.pem)"
MAIN_SUMMARY_PROMPT_WORDLIST = "Please enter your wordlist (.txt)"
MAIN_SUMMARY_PROMPT_KEY = "Please enter your key"
MAIN_SUMMARY_PROMPT_INJECTION = "Please enter your injection"
MAIN_SUMMARY_PROMPT_JWKS = "Url of your jwks is stored (your external ip)"
# modify_summary
MODIFY_SUMMARY_QUESTION = "What do you want to do?"

SEPARATOR_HEADER = Separator(
    "---------------------------- Header ----------------------------",
)
SEPARATOR_PAYLOAD = Separator(
    "---------------------------- Payload ----------------------------",
)
MODIFY_SUMMARY_CHOICES_ADD_HEADER = "add header value"
MODIFY_SUMMARY_CHOICES_ADD_PAYLOAD = "add payload value"
MODIFY_SUMMARY_CHOICES_RETURN = "Return"

MODIFY_SUMMARY_PROMPT_VALUE = "Please enter a value"
MODIFY_SUMMARY_PROMPT_KEY = "Please enter a key"



"""Test"""
import os

import pytest as pytest

from myjwt.Exception import InvalidJWT
from myjwt.Exception import InvalidJwtJson
from myjwt.utils import create_crt
from myjwt.utils import encode_jwt
from myjwt.utils import encoded_to_json
from myjwt.utils import HEADER
from myjwt.utils import is_valid_jwt
from myjwt.utils import is_valid_jwt_json
from myjwt.utils import jwt_to_json
from myjwt.utils import PAYLOAD
from myjwt.utils import SIGNATURE

invalid_jwt = "test.test"
jwt = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0.eyJsb2dpbiI6ImF6In0."
encoded_string = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0"
header = {"typ": "JWT", "alg": "none"}
payload = {"login": "az"}
signature = ""
jwt_json = {
    HEADER: header,
    PAYLOAD: payload,
    SIGNATURE: signature,
}


def test_jwt_to_json_InvalidJWT():
    """
    Test jwt_to_json method when jwt is invalid in utils.py
    """
    with pytest.raises(InvalidJWT):
        jwt_to_json(invalid_jwt)


def test_jwt_to_json():
    """
    Test jwt_to_json method in utils.py
    """
    jwt_json = jwt_to_json(jwt)
    assert type(jwt_json) == dict

    assert list(jwt_json.keys()) == [HEADER, PAYLOAD, SIGNATURE]

    assert jwt_json[HEADER] == header
    assert jwt_json[PAYLOAD] == payload
    assert jwt_json[SIGNATURE] == ""


def test_encoded_to_json():
    """
    Test encoded_to_json method in utils.py
    """
    jsonDecoded = encoded_to_json(encoded_string)

    assert type(jsonDecoded) == dict
    assert jsonDecoded == header


def test_encode_jwt():
    """
    Test encode_jwt method in utils.py
    """
    with pytest.raises(InvalidJwtJson):
        encode_jwt({})

    new_jwt = encode_jwt(jwt_json)
    assert new_jwt + "." == jwt


def test_is_valid_jwt():
    """
    Test is_valid_jwt method in utils.py
    """
    assert is_valid_jwt(jwt)


def test_is_valid_jwt_json():
    """
    Test is_valid_jwt_json method in utils.py
    """
    assert is_valid_jwt_json(jwt_json)


def test_create_crt():
    """
    Test create_crt method in utils.py
    """
    create_crt()
    assert os.path.exists("selfsigned.crt")
    assert os.path.exists("private.pem")



"""User interface file"""
import re
from typing import Dict

import click
import questionary

from myjwt.modify_jwt import signature
from myjwt.utils import copy_to_clipboard
from myjwt.utils import encode_jwt
from myjwt.utils import HEADER
from myjwt.utils import jwt_to_json
from myjwt.utils import PAYLOAD
from myjwt.utils import SIGNATURE
from myjwt.variables import CHECK_DOCS
from myjwt.variables import CRACKED
from myjwt.variables import custom_style_fancy
from myjwt.variables import INVALID_SIGNATURE
from myjwt.variables import MAIN_SUMMARY_CHOICES
from myjwt.variables import MAIN_SUMMARY_CHOICES_BRUTE_FORCE
from myjwt.variables import MAIN_SUMMARY_CHOICES_JKU
from myjwt.variables import MAIN_SUMMARY_CHOICES_KID
from myjwt.variables import MAIN_SUMMARY_CHOICES_MODIFY
from myjwt.variables import MAIN_SUMMARY_CHOICES_NONE_ALG
from myjwt.variables import MAIN_SUMMARY_CHOICES_QUIT
from myjwt.variables import MAIN_SUMMARY_CHOICES_RSA_CONFUSION
from myjwt.variables import MAIN_SUMMARY_CHOICES_SIGN
from myjwt.variables import MAIN_SUMMARY_CHOICES_VERIFY
from myjwt.variables import MAIN_SUMMARY_CHOICES_X5U
from myjwt.variables import MAIN_SUMMARY_PROMPT_INJECTION
from myjwt.variables import MAIN_SUMMARY_PROMPT_JWKS
from myjwt.variables import MAIN_SUMMARY_PROMPT_KEY
from myjwt.variables import MAIN_SUMMARY_PROMPT_PEM
from myjwt.variables import MAIN_SUMMARY_PROMPT_WORDLIST
from myjwt.variables import MAIN_SUMMARY_QUESTION
from myjwt.variables import MODIFY_SUMMARY_CHOICES_ADD_HEADER
from myjwt.variables import MODIFY_SUMMARY_CHOICES_ADD_PAYLOAD
from myjwt.variables import MODIFY_SUMMARY_CHOICES_RETURN
from myjwt.variables import MODIFY_SUMMARY_PROMPT_KEY
from myjwt.variables import MODIFY_SUMMARY_PROMPT_VALUE
from myjwt.variables import MODIFY_SUMMARY_QUESTION
from myjwt.variables import NEW_JWT
from myjwt.variables import NOT_CRAKED
from myjwt.variables import SEPARATOR_HEADER
from myjwt.variables import SEPARATOR_PAYLOAD
from myjwt.variables import VALID_SIGNATURE
from myjwt.vulnerabilities import bruteforce_wordlist
from myjwt.vulnerabilities import confusion_rsa_hmac
from myjwt.vulnerabilities import inject_sql_kid
from myjwt.vulnerabilities import jku_vulnerability
from myjwt.vulnerabilities import none_vulnerability
from myjwt.vulnerabilities import print_decoded


def user_interface(jwt: str) -> None:
    """
    User interface for myjwt.
    Print decoded jwt then list choices of vulnerabilities.

    Parameters
    ----------
    jwt: Your jwt
    """
    click.echo("")
    click.echo("Your jwt is: \n" + jwt)
    click.echo("")
    click.echo("Your jwt decoded is:")
    click.echo("")
    print_decoded(jwt)
    click.echo("")
    jwt_json = jwt_to_json(jwt)
    summary = ""
    while summary != MAIN_SUMMARY_CHOICES_QUIT and summary is not None:
        summary = questionary.select(
            MAIN_SUMMARY_QUESTION,
            choices=MAIN_SUMMARY_CHOICES,
            style=custom_style_fancy,
        ).ask()
        if summary == MAIN_SUMMARY_CHOICES_MODIFY:
            jwt_json = user_modify_jwt(jwt_json)
        elif summary == MAIN_SUMMARY_CHOICES_NONE_ALG:
            user_none_vulnerability(jwt_json)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_RSA_CONFUSION:
            hmac = click.prompt(
                MAIN_SUMMARY_PROMPT_PEM,
                type=click.Path(exists=True),
            )
            user_confusion_rsa_hmac(jwt_json, hmac)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_BRUTE_FORCE:
            wordlist = click.prompt(
                MAIN_SUMMARY_PROMPT_WORDLIST,
                type=click.Path(exists=True),
            )
            user_bruteforce_wordlist(jwt_json, wordlist)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_SIGN:
            key = click.prompt(MAIN_SUMMARY_PROMPT_KEY, type=str)
            user_sign_jwt(jwt_json, key)
            summary = ""
        elif summary == MAIN_SUMMARY_CHOICES_VERIFY:
            key = click.prompt(MAIN_SUMMARY_PROMPT_KEY, type=str)
            user_verify_key(jwt_json, key)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_KID:
            injection = click.prompt(MAIN_SUMMARY_PROMPT_INJECTION, type=str)
            new_jwt = user_kid_injection(jwt_json, injection)
            jwt_json = jwt_to_json(new_jwt)
            click.echo(NEW_JWT + new_jwt)
            copy_to_clipboard(new_jwt)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_JKU:
            url = click.prompt(MAIN_SUMMARY_PROMPT_JWKS, type=str)
            user_jku_by_pass(jwt_json, url)
            summary = MAIN_SUMMARY_CHOICES_QUIT
        elif summary == MAIN_SUMMARY_CHOICES_X5U:
            url = click.prompt(MAIN_SUMMARY_PROMPT_JWKS, type=str)
            user_x5u_by_pass(jwt_json, url)
            summary = MAIN_SUMMARY_CHOICES_QUIT


def user_modify_jwt(jwt_json: Dict) -> Dict:
    """
    Print for modify interface

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).

    Returns
    -------
    Dict
        jwt Dict
    """
    item = ""
    while item != MODIFY_SUMMARY_CHOICES_RETURN and item is not None:
        header_list = list()
        for key in jwt_json[HEADER].keys():
            header_list.append(
                str(key)
                + " = "
                + (
                    str(jwt_json[HEADER][key])
                    if jwt_json[HEADER][key] is not None
                    else "null"
                ),
            )

        payload_list = list()
        for key in jwt_json[PAYLOAD].keys():
            payload_list.append(
                str(key)
                + " = "
                + (
                    str(jwt_json[PAYLOAD][key])
                    if jwt_json[PAYLOAD][key] is not None
                    else "null"
                ),
            )
        item = questionary.select(
            MODIFY_SUMMARY_QUESTION,
            choices=[SEPARATOR_HEADER]
            + header_list
            + [MODIFY_SUMMARY_CHOICES_ADD_HEADER]
            + [SEPARATOR_PAYLOAD]
            + payload_list
            + [
                MODIFY_SUMMARY_CHOICES_ADD_PAYLOAD,
                MODIFY_SUMMARY_CHOICES_RETURN,
            ],
            style=custom_style_fancy,
        ).ask()
        if item in header_list:
            m = re.match("(.*) = .*", item)
            key = m.groups()[0]  # type: ignore
            value = click.prompt(MODIFY_SUMMARY_PROMPT_VALUE, type=str)
            jwt_json[HEADER][key] = value
        elif item in payload_list:
            m = re.match("(.*) = .*", item)
            key = m.groups()[0]  # type: ignore
            value = click.prompt(MODIFY_SUMMARY_PROMPT_VALUE, type=str)
            jwt_json[PAYLOAD][key] = value
        elif item == MODIFY_SUMMARY_CHOICES_ADD_HEADER:
            key = click.prompt(MODIFY_SUMMARY_PROMPT_KEY, type=str)
            value = click.prompt(MODIFY_SUMMARY_PROMPT_VALUE, type=str)
            jwt_json[HEADER][key] = value
        elif item == MODIFY_SUMMARY_CHOICES_ADD_PAYLOAD:
            key = click.prompt(MODIFY_SUMMARY_PROMPT_KEY, type=str)
            value = click.prompt(MODIFY_SUMMARY_PROMPT_VALUE, type=str)
            jwt_json[PAYLOAD][key] = value
    return jwt_json


def user_none_vulnerability(jwt_json: Dict) -> None:
    """
    Print for none vulnerability.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    """
    jwt = none_vulnerability(encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE])
    copy_to_clipboard(jwt)
    click.echo(NEW_JWT + jwt)


def user_confusion_rsa_hmac(jwt_json: Dict, hmac: str) -> None:
    """
    Print for rsa/hmac confusion.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    hmac: str
        path of your public key.
    """
    jwt = confusion_rsa_hmac(
        encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE],
        hmac,
    )
    copy_to_clipboard(jwt)
    click.echo(NEW_JWT + jwt)


def user_bruteforce_wordlist(jwt_json: Dict, wordlist: str) -> None:
    """
    Print For bruteforce method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    wordlist: str
        path of your wordlist
    """
    if "HS" not in jwt_json[HEADER]["alg"]:
        click.echo(CHECK_DOCS)
    key = bruteforce_wordlist(
        encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE],
        wordlist,
    )
    if key == "":
        click.echo(NOT_CRAKED)
    else:
        copy_to_clipboard(key)
        click.echo(CRACKED + key)


def user_verify_key(jwt_json: Dict, key: str) -> None:
    """
     Print For verify method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    key: str
        your key
    """
    if "HS" not in jwt_json[HEADER]["alg"]:
        click.echo(CHECK_DOCS)
    new_jwt = signature(jwt_json, key)
    click.echo(
        VALID_SIGNATURE
        if new_jwt.split(".")[2] == jwt_json[SIGNATURE]
        else INVALID_SIGNATURE,
    )


def user_sign_jwt(jwt_json: Dict, key: str) -> None:
    """
     Print For sign method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    key: str
        your key
    """
    if "HS" not in jwt_json[HEADER]["alg"]:
        click.echo(CHECK_DOCS)
    jwt = signature(jwt_json, key)
    copy_to_clipboard(jwt)
    click.echo(NEW_JWT + jwt)


def user_kid_injection(jwt_json: Dict, injection: str) -> str:
    """
    Print for kid injection method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    injection: str
        your injection

    Returns
    ----------
    str
        Your jwt.
    """
    return inject_sql_kid(
        jwt=encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE],
        injection=injection,
    )


def user_jku_by_pass(jwt_json: Dict, url: str) -> None:
    """
    Print for jku bypass method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    url: str
        your url
    """
    new_jwt = jku_vulnerability(
        jwt=encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE],
        url=url,
    )
    click.echo(NEW_JWT + new_jwt)
    copy_to_clipboard(new_jwt)
    click.echo(
        f"Please run python -m http.server --bind {url} .Before send your jwt",
    )


def user_x5u_by_pass(jwt_json: Dict, url: str) -> None:
    """
    Print for x5u bypass method.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    url: str
        your url
    """
    new_jwt = jku_vulnerability(
        jwt=encode_jwt(jwt_json) + "." + jwt_json[SIGNATURE],
        url=url,
    )
    click.echo(NEW_JWT + new_jwt)
    copy_to_clipboard(new_jwt)
    click.echo(
        f"Please run python -m http.server --bind {url} .Before send your jwt",
    )



"""
Package for modify your jwt(header, payload, signature)
"""
import base64
import hashlib
import hmac
from typing import Dict

from myjwt.Exception import InvalidJwtJson
from myjwt.Exception import InvalidParam
from myjwt.Exception import UnknownAlg
from myjwt.utils import encode_jwt
from myjwt.utils import HEADER
from myjwt.utils import is_valid_jwt_json
from myjwt.utils import PAYLOAD


def add_payload(jwt_json: Dict, payload) -> Dict:
    """
    Add new key:value to jwt's payload.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    payload: Dict
        add value to your payload.

    Returns
    -------
    Dict
        a new jwt in json format.

    Raises
    -------
    InvalidJwtJson
        if your jwt_json is not a Dict.

    InvalidParam
        if your payload is not a Dict.
    """
    if not is_valid_jwt_json(jwt_json):
        raise InvalidJwtJson("Invalid JWT json format")

    if type(payload) is not dict:
        raise InvalidParam("Invalid Payload")

    for payload_key in payload.keys():
        jwt_json[PAYLOAD][payload_key] = payload[payload_key]
    return jwt_json


def add_header(jwt_json: Dict, header: Dict) -> Dict:
    """
    Add new key:value to jwt's header.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    header: Dict
        add value to your header.

    Returns
    -------
    Dict
        a new jwt in json format.

    Raises
    -------
    InvalidJwtJson
        if your jwt_json is not a Dict.

    InvalidParam
        if your header is not a Dict.
    """
    if not is_valid_jwt_json(jwt_json):
        raise InvalidJwtJson("Invalid JWT json format")

    if type(header) is not dict:
        raise InvalidParam("Invalid Header")

    for header_key in header.keys():
        jwt_json[HEADER][header_key] = header[header_key]
    return jwt_json


def change_alg(jwt_json: Dict, algo: str) -> Dict:
    """
    Change alg of your jwt.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    algo: str
        new algo.

    Returns
    -------
    Dict
        a new jwt in json format.

    Raises
    -------
    InvalidJwtJson
        if your jwt_json is not a Dict.
    """
    if not is_valid_jwt_json(jwt_json):
        raise InvalidJwtJson("Invalid JWT json format")

    jwt_json[HEADER]["alg"] = algo
    return jwt_json


def change_payload(jwt_json: Dict, payload: Dict) -> Dict:
    """
    Change the current payload to your jwt_json for the new payload given.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    payload: Dict
        new payload

    Returns
    -------
    Dict
        a new jwt in json format.

    Raises
    -------
    InvalidJwtJson
        if your jwt_json is not a Dict.
    """
    if not is_valid_jwt_json(jwt_json):
        raise InvalidJwtJson("Invalid JWT json format")
    jwt_json[PAYLOAD] = payload
    return jwt_json


def signature(jwt_json: Dict, key: str) -> str:
    """
    Sign your jwt.

    Parameters
    ----------
    jwt_json: Dict
        your jwt json (use encode_to_json.Check Doc).
    key: str
        key for dign your new jwt.

    Returns
    -------
    str
        new jwt.

    Raises
    -------
    InvalidJwtJson
        if your jwt_json is not a Dict.

    UnknownAlg
        if your alg is not a valid alg. Accepted: none, HS{256,384,512}.
    """
    if not is_valid_jwt_json(jwt_json):
        raise InvalidJwtJson("Invalid JWT json format")

    if jwt_json[HEADER]["alg"] == "none":
        return encode_jwt(jwt_json) + "."
    elif jwt_json[HEADER]["alg"] == "HS256":
        jwt = encode_jwt(jwt_json)
        signature_hmac = hmac.new(
            key.encode(),
            jwt.encode(),
            hashlib.sha256,
        ).digest()
        new_signature = (
            base64.urlsafe_b64encode(signature_hmac).decode("UTF-8").strip("=")
        )
        return jwt + "." + new_signature
    elif jwt_json[HEADER]["alg"] == "HS384":
        jwt = encode_jwt(jwt_json)
        signature_hmac = hmac.new(
            key.encode(),
            jwt.encode(),
            hashlib.sha384,
        ).digest()
        new_signature = (
            base64.urlsafe_b64encode(signature_hmac).decode("UTF-8").strip("=")
        )
        return jwt + "." + new_signature
    elif jwt_json[HEADER]["alg"] == "HS512":
        jwt = encode_jwt(jwt_json)
        signature_hmac = hmac.new(
            key.encode(),
            jwt.encode(),
            hashlib.sha512,
        ).digest()
        new_signature = (
            base64.urlsafe_b64encode(signature_hmac).decode("UTF-8").strip("=")
        )
        return jwt + "." + new_signature

    raise UnknownAlg(
        "Unknown alg " + jwt_json[HEADER]["alg"] + "send an issue please.",
    )



"""Test"""
import json
import re
from io import UnsupportedOperation

import requests
from click.testing import CliRunner

from myjwt.modify_jwt import change_payload
from myjwt.myjwt_cli import myjwt_cli
from myjwt.utils import HEADER
from myjwt.utils import jwt_to_json
from myjwt.utils import PAYLOAD
from myjwt.utils import SIGNATURE
from myjwt.variables import CHECK_DOCS
from myjwt.variables import NOT_VALID_JWT
from myjwt.variables import VALID_PAYLOAD_JSON

test_jwt = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJsb2dpbiI6ImEifQ.Fjziy6GSQpP9tQRyko5APZjdymkQ8EJGOa"
    "-A2JQ6xcAVucXRhZbdBbAM2DG8io_brP_ROAqYaNlvRVsztXoPHFz_e7D2K0q6f02RXeRwZJGOhy0K"
    "-Oj9Z1UmFJWqVpAAafN75w7OKoSRh6BtQfH8XDleqwpVoywCuWFdYrSbqBoVskRQkp8H-HUC5XmN5om4"
    "-NdiQkiKa7OFQ6Hoklclz9_WD5rc"
    "-HWJp3rJW4EIHzOPfs1GuDuhtIRu0uuRYp4vvzLZcVm0BhlK9e_fmFcbsTz3MwVHIeFEIx2NjQdhE"
    "-CefQ4tNg6Rr6OtgGExToUfD0i0mAoAhTcvmoyO6c2paQ"
)
jwt_bruteforce = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJ1c2VyIjpudWxsfQ"
    ".Tr0VvdP6rVBGBGuI_luxGCOaz6BbhC6IxRTlKOW8UjM"
)
password_path = "./wordlist/common_pass.txt"
jwt_kid = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiIsImtpZCI6ImtleTEifQ.eyJ1c2VyIjpudWxsfQ"
    ".2B9ZKzJ3FeJ9yoNLDGKgcxOuo05PwDRzFQ_34CrGteQ"
)
kid_injection = "../../../../../../dev/null"

jwt_jku = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImprdSI6Imh0dHA6Ly9sb2NhbGhvc3Q6ODA4MCJ9.eyJ1c2VyIjoiYSJ9"
    ".e1oZ73Q95aYPcRfulEY--beuGEV2tE1W_FGHtH1ZlevC76lBVqbdM5PY1v6quuJWRtNLwqDbUdydAH4lubgE0pwix-A7LqcD-b"
    "-0mNQkt9jXqBYCYBsZtGnvBFB9qHoK_CI39qLku1rOWkcEOcJYMSJFfxipImBb_AwoiXv"
    "-wmnpchTOAY_PFOtXVXKHkoGQtEaMKfnRBXHAgyEAcqHCqvljWuMmdKVpyGNVaQBnKCEKkGyYLpdpL2UIZ3XNYy96JcGpm6LHvIXm6r"
    "EOkWoJl2j_07xVsM2S__QzllRw_qezS5rzuYlRz-0j0nP_S5gSRcdrR4yNtSO3ivue5mR-RQ"
)

jwt_x5u = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsIng1dSI6Imh0dHA6Ly90ZXN0LmNvbSJ9.eyJ1c2VyIjoiaGFja2VyIn0.Z57BGf-BW"
    "WGCYGRST3PstC7dqFVxLpYh8D9iy6z8_tpz8vIESa5IdLt3hkM8ysB0IjrkWbgNMYTaP7YiGpHG7MhF_IAc_q8HOilMtvrVTyJ0EpE3uJ"
    "okXZSh_hhU5ay2K8H743AG_5x7coAf7ZsNe_rnSuDN6iV_oXo31H2ga9VMk2BLgvqFLYgIYVETeQbcSx4M2rxiH20VbqO4dwzYDedYkD"
    "AHKGHUAI0eXJoJ7Sq3sDrjZ9_THTiHSwQQYFnlIbIcFKuANdExuhG-tmIhfa6-8Zu_RELLL6UzgL2G-yu021B_Hm9YmwuXewtDktXKY"
    "uWofo-PVFUUWVSEw7gIAw"
)


def test_error_cli():
    """
    Test error input during call in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [])
    assert result.exit_code == 2

    result = CliRunner().invoke(myjwt_cli, ["Peter"])
    assert NOT_VALID_JWT in result.output
    assert result.exit_code == 1


def test_payload():
    """
    Test Payload option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--full-payload"])
    assert result.exit_code, 2

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--full-payload", "test"],
    )
    assert VALID_PAYLOAD_JSON in result.output

    result = CliRunner().invoke(
        myjwt_cli,
        [
            test_jwt,
            "--full-payload",
            '{"username": "test", "password": "test"}',
        ],
    )
    jwtVerify = change_payload(
        jwt_to_json(test_jwt),
        json.loads('{"username": "test", "password": "test"}'),
    )
    jwt = re.search("new JWT: " + "(.*)", result.output).groups()[0]
    assert jwt_to_json(jwt) == jwtVerify
    assert result.exit_code == 0


def test_add_header():
    """
    Test add-header option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--add-header"])
    assert result.exit_code == 2

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--add-header", "admin"],
    )
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--add-header", "username=admin"],
    )
    jwt = re.search("new JWT: " + "(.*)", result.output).groups()[0]
    jwt_json = jwt_to_json(jwt)
    assert jwt_json[HEADER]["username"] == "admin"
    assert result.exit_code == 0


def test_add_payload():
    """
    Test add-payload option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--add-payload"])
    assert result.exit_code == 2

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--add-payload", "admin"],
    )
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--add-payload", "username=admin"],
    )
    jwt = re.search("new JWT: " + "(.*)", result.output).groups()[0]
    jwt_json = jwt_to_json(jwt)
    assert jwt_json[PAYLOAD]["username"] == "admin"
    assert result.exit_code == 0


def test_sign():
    """
    Test sign option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--sign"])
    assert result.exit_code == 2

    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--sign", "test"])
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_bruteforce, "--sign", "pentesterlab"],
    )
    jwt = re.search("new JWT: " + "(.*)", result.output).groups()[0]
    assert jwt_bruteforce == jwt
    assert result.exit_code == 0


def test_verify():
    """
    Test verify option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--verify"])
    assert result.exit_code == 2

    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--verify", "test"])
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_bruteforce, "--verify", "pentesterlab"],
    )
    assert "Valid Signature!!" in result.output
    assert result.exit_code == 0


def test_none_vulnerability():
    """
    Test none-vulnerability option in myjwt_cli.py
    """
    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--none-vulnerability"],
    )
    jwt = re.search("new JWT: " + "(.*)", result.output).groups()[0]
    assert "none" == jwt_to_json(jwt)[HEADER]["alg"]
    assert "" == jwt_to_json(jwt)[SIGNATURE]
    assert result.exit_code == 0


def test_hmac():
    """
    Test hmac option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--hmac"])
    assert result.exit_code == 2

    result = CliRunner().invoke(
        myjwt_cli,
        [
            test_jwt,
            "--hmac",
            "./examples/05-rsa-hmac-confusion/public.pem",
        ],
    )
    assert result.exit_code == 0


def test_bruteforce():
    """
    Test bruteforce option in myjwt_cli.py
    """
    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--bruteforce", "azdzd"],
    )
    assert result.exit_code, 2

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "--bruteforce", password_path],
    )
    assert CHECK_DOCS in result.output
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_bruteforce, "--bruteforce", "./wordlist/empty.txt"],
    )
    assert "JWT not cracked sorry. " in result.output
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_bruteforce, "--bruteforce", password_path],
    )
    assert "JWT cracked, key is: " + "pentesterlab" in result.output
    assert result.exit_code == 0


def test_kid():
    """
    Test kid option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--kid"])
    assert result.exit_code == 2

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_kid, "--kid", kid_injection],
    )
    assert result.exit_code == 0


def test_print():
    """
    Test print option in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt, "--print"])
    assert result.exit_code == 0


def test_url(requests_mock):
    """
    Test url option in myjwt_cli.py
    """
    status_code = 200
    requests_mock.get(
        "http://localhost:8080",
        json={},
        status_code=status_code,
    )

    result = CliRunner().invoke(
        myjwt_cli,
        [
            test_jwt,
            "-u",
            "http://localhost:8080",
            "-c",
            "data=data",
            "-d",
            "data=data",
        ],
    )
    assert result.exit_code == 0

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "-u", "http://localhost:8080", "-c", "data"],
    )
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "-u", "http://localhost:8080", "-d", "data"],
    )
    assert result.exit_code == 1

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "-u", "http://localhost:8080", "-d", "data=MY_JWT"],
    )
    assert result.exit_code == 0

    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "-u", "http://localhost:8080", "-c", "data=MY_JWT"],
    )
    assert result.exit_code == 0


def test_url_connectionError(mocker):
    """
    Test url error for url option in myjwt_cli.py
    """
    mocker.patch(
        "myjwt.vulnerabilities.send_jwt_to_url",
        side_effect=requests.exceptions.ConnectionError,
    )
    result = CliRunner().invoke(
        myjwt_cli,
        [test_jwt, "-u", "http://www.azdazdazdzadazdazdzad.com"],
    )
    assert "Connection Error. Verify your url" in result.output
    assert result.exit_code == 1


def test_jku(requests_mock):
    """
    Test jku option in myjwt_cli.py
    """
    status_code = 200
    requests_mock.get(
        "http://localhost:8080",
        json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                    "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                    "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                    "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256",
                },
            ],
        },
        status_code=status_code,
    )

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_jku, "--jku", "http://localhost:8080"],
    )
    assert result.exit_code == 0


def test_user_interface():
    """
    Test user_interface in myjwt_cli.py
    """
    result = CliRunner().invoke(myjwt_cli, [test_jwt])
    # raise UnsupportedOperation(stdin is not a terminal)
    assert type(result.exception) == UnsupportedOperation


def test_x5c(requests_mock):
    """
    Test x5c option in myjwt_cli.py
    """
    status_code = 200
    requests_mock.get(
        "http://test.com",
        json={
            "keys": [
                {
                    "kty": "RSA",
                    "use": "sig",
                    "kid": "xxxxxxxxx",
                    "n": "oTtAXRgdJ6Pu0jr3hK3opCF5uqKWKbm4KkqIiDJSEsQ4PnAz14P_aJnfnsQwgchFGN95cfCO7euC8HjT"
                    "-u5WHHDn08GQ7ot6Gq6j-fbwMdRWjLC74XqQ0JNDHRJoM4bbj4i8FaBdYKvKmnJ8eSeEjA0YrG8KuTOPbLsgl"
                    "ADUubNw9kggRIvj6au88dnBJ9HeZ27QVVFaIllZpMITtocuPkOKd8bHzkZzKN4HJtM0hgzOjeyCfqZxh1V8LybliWD"
                    "XYivUqmvrzchzwXTAQPJBBfYo9BO6D4Neui8rGbc49OBCnHLCWtPH7m7xp3cz-PbVnLhRczzsQE_3escvTF0FGw",
                    "e": "AQAB",
                    "alg": "RS256",
                },
            ],
        },
        status_code=status_code,
    )

    result = CliRunner().invoke(
        myjwt_cli,
        [jwt_x5u, "--x5u", "http://test.com"],
    )
    assert result.exit_code == 0



"""Test"""
import pytest

from myjwt.Exception import InvalidJwtJson
from myjwt.Exception import InvalidParam
from myjwt.Exception import UnknownAlg
from myjwt.modify_jwt import add_header
from myjwt.modify_jwt import add_payload
from myjwt.modify_jwt import change_alg
from myjwt.modify_jwt import change_payload
from myjwt.modify_jwt import signature
from myjwt.utils import HEADER
from myjwt.utils import jwt_to_json
from myjwt.utils import PAYLOAD
from myjwt.utils import SIGNATURE

invalid_jwt = "test.test"
jwt = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0.eyJsb2dpbiI6ImF6In0."
jwt_rsa = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiJ9.eyJsb2dpbiI6ImEifQ.Fjziy6GSQpP9tQRyko5APZjdymkQ8EJGOa"
    "-A2JQ6xcAVucXRhZbdBbAM2DG8io_brP_ROAqYaNlvRVsztXoPHFz_e7D2K0q6f02RXeRwZJGOhy0K"
    "-Oj9Z1UmFJWqVpAAafN75w7OKoSRh6BtQfH8XDleqwpVoywCuWFdYrSbqBoVskRQkp8H-HUC5XmN5om4"
    "-NdiQkiKa7OFQ6Hoklclz9_WD5rc"
    "-HWJp3rJW4EIHzOPfs1GuDuhtIRu0uuRYp4vvzLZcVm0BhlK9e_fmFcbsTz3MwVHIeFEIx2NjQdhE"
    "-CefQ4tNg6Rr6OtgGExToUfD0i0mAoAhTcvmoyO6c2paQ"
)
jwt_hs256 = (
    "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJsb2dpbiI6ImEifQ.KJDuTWSj9wa3NL3j1u2HOijvgu"
    "-oO9tBjKGxjo_qdXQ"
)
encoded_string = "eyJ0eXAiOiJKV1QiLCJhbGciOiJub25lIn0"
header = {"typ": "JWT", "alg": "none"}
add_header_value = {"kid": "1"}
payload = {"login": "az"}
add_payload_value = {"username": "az"}
signature_value = ""
jwt_json = {
    HEADER: header,
    PAYLOAD: payload,
    SIGNATURE: signature_value,
}
path = "./examples/05-rsa-hmac-confusion/public.pem"


def test_add_payload():
    """
    Test add_payload method in modify_jwt.py
    """
    with pytest.raises(InvalidJwtJson):
        add_payload({}, {})

    with pytest.raises(InvalidParam):
        add_payload(jwt_json, "")

    new_jwt_json = add_payload(jwt_json, add_payload_value)
    assert list(new_jwt_json[PAYLOAD].keys()) == ["login", "username"]
    assert new_jwt_json[PAYLOAD]["username"] == add_payload_value["username"]
    assert new_jwt_json[PAYLOAD]["login"] == payload["login"]

    assert new_jwt_json[HEADER] == jwt_json[HEADER]
    assert new_jwt_json[SIGNATURE] == jwt_json[SIGNATURE]


def test_add_header():
    """
    Test add_header method in modify_jwt.py
    """
    with pytest.raises(InvalidJwtJson):
        add_header({}, {})

    with pytest.raises(InvalidParam):
        add_header(jwt_json, "{}")

    new_jwt_json = add_header(jwt_json, add_header_value)
    assert list(new_jwt_json[HEADER].keys()), ["typ", "alg", "kid"]
    assert new_jwt_json[HEADER]["typ"] == header["typ"]
    assert new_jwt_json[HEADER]["alg"] == header["alg"]
    assert new_jwt_json[HEADER]["kid"] == add_header_value["kid"]

    assert new_jwt_json[PAYLOAD] == jwt_json[PAYLOAD]
    assert new_jwt_json[SIGNATURE] == jwt_json[SIGNATURE]


def test_change_alg():
    """
    Test changeAlg method in modify_jwt.py
    """
    with pytest.raises(InvalidJwtJson):
        change_alg({}, "test")

    new_jwt_json = change_alg(jwt_json, "test")
    assert new_jwt_json[HEADER]["alg"], "test"

    assert new_jwt_json[PAYLOAD] == jwt_json[PAYLOAD]
    assert new_jwt_json[SIGNATURE] == jwt_json[SIGNATURE]


def test_change_payload():
    """
    Test changePayload method in modify_jwt.py
    """
    with pytest.raises(InvalidJwtJson):
        change_payload({}, {})

    new_jwt_json = change_payload(jwt_json, add_payload_value)

    assert new_jwt_json[PAYLOAD] == jwt_json[PAYLOAD]

    assert new_jwt_json[HEADER] == jwt_json[HEADER]
    assert new_jwt_json[SIGNATURE] == jwt_json[SIGNATURE]


def test_signature():
    """
    Test signature method in modify_jwt.py
    """
    with pytest.raises(InvalidJwtJson):
        signature({}, "")

    jwt_json = {
        HEADER: {"typ": "JWT", "alg": "none"},
        PAYLOAD: {"login": "az"},
        SIGNATURE: "",
    }
    new_jwt = signature(jwt_json, "")
    assert new_jwt == jwt

    jwt_json_test = jwt_to_json(jwt_rsa)
    jwt_json_test = change_alg(jwt_json_test, "HS256")
    new_jwt = signature(jwt_json_test, open(path).read())
    assert new_jwt == jwt_hs256

    new_jwt_json = jwt_json
    new_jwt_json[HEADER]["alg"] = "unknowAlg"
    with pytest.raises(UnknownAlg):
        signature(new_jwt_json, "")



# type: ignore
import re
import subprocess

import setuptools

version = re.search(
    r'^__version__\s*=\s*"(.*)"',
    open("myjwt/__init__.py").read(),
    re.M,
).group(
    1,
)

try:
    commit = subprocess.check_output("git rev-parse --verify HEAD", shell=True)
except subprocess.CalledProcessError as e:
    print("Exception on process, rc=", e.returncode, "output=", e.output)
    commit = b"commit name not found"

with open("myjwt/__init__.py", "w") as f:
    f.write(
        f'"""autogenerated"""\n__version__ = "{str(version)}"\n__commit__ = "{str(commit, "utf-8").strip()}"\n',
    )
f.close()

with open("README.md") as f:
    long_description = f.read()
f.close()

with open("dev-requirements.txt") as f:
    dev_requires = [line.strip() for line in f]
    f.close()
with open("docs/requirements.txt") as f:
    docs_requires = [line.strip() for line in f]
    f.close()

with open("requirements.txt") as f:
    install_requires = [line.strip() for line in f]
    f.close()

setuptools.setup(
    name="myjwt",
    version=version,
    author="mBouamama",
    author_email="matthieubouamama@gmail.com",
    description="Pentesting Tool for JWT(JSON Web Tokens).Modify/Crack/Check Your jwt.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/mBouamama/MyJWT",
    install_requires=install_requires,
    entry_points={"console_scripts": ["myjwt = myjwt.myjwt_cli:myjwt_cli"]},
    extras_require={"dev": dev_requires, "docs": docs_requires},
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Security",
        "Topic :: Internet :: WWW/HTTP",
    ],
    python_requires=">=3.6",
)
